/* ----------------------------------------------------------
                         viergewinnt.c

     Vier gewinnt fuer AVR-ATmega mit vereinfachter
     (was bedeutet: nicht so guter) KI.

     Aufgrund nicht genuegenden RAM konnte ein Mini-Max-
     Algorithmus nicht implementiert werden.

     MCU   :  ATmega328
     Takt  :  16 MHz

     Zusaetzliche Hardware:
              128x128 TFT-Display
              4 Tasten

     03.06.2019  R. Seelig
   ---------------------------------------------------------- */

#include <stdint.h>
#include <stdlib.h>
#include <string.h>

#include <avr/io.h>

#include "avr_gpio.h"
#include "tftdisplay.h"
#include "gfx_pictures.h"

#include "vg_logo.h"

#define delay   _delay_ms
#define my_outtextxy(x,y,ptr)   outtextxy(x,y,0,ptr)

#define  empty        0
#define  human        1
#define  cpu          2
#define  outside      3

#define xofs   0
#define yofs   20

// ----------------- Spieltasten ---------------

#define keyright_init() PD6_input_init()      // Taste rechts
#define keyup_init()    PD3_input_init()      // Taste oben
#define keyleft_init()  PD4_input_init()      // Taste links
#define keydwn_init()   PD5_input_init()      // Taste runter

#define is_keyright()   ( !(is_PD6()) )
#define is_keyup()      ( !(is_PD3()) )
#define is_keyleft()    ( !(is_PD4()) )
#define is_keydwn()     ( !(is_PD5()) )

#define button_init()  { keydwn_init(); keyleft_init(); keyright_init(); keyup_init() }

// --------------- globale Variable ------------

uint8_t  c4_field [7][6];             // Spielfeldarray
int8_t   colf [7];                    // Fuellhoehe der einzelnen Spalten
uint8_t  c4_winner;                   //

uint16_t random_startwert;
char     selectpos = 3;

uint8_t  playerscore = 0,
         cpuscore = 0;

/* --------------------------------------------------------
                         my_random

    generiert Pseudozufallszahl von 0 .. 3 (inklusive) die
    mittels mitgekoppeltem Schieberegister erreicht wird

    Uebergabe:
      startwert  : Ausgangspunkt im Zahlenbereich

    rndm_mask ist die XOR-Maske beim Mitkoppeln des
    Schieberegisters

          XOR-Maske nach     Anzahl unterschiedlichen
          dem Schieben       Zahlen im Ergebnis
          -------------------------------------------
              0xb400               65535
              0x07c1                2047
              0x0be0                4095
  -------------------------------------------------------- */
uint16_t my_random(uint16_t rmax)
{
  // Variable MUESSEN static sein, damit das Schieberegister
  // mit jedem Tick von der vorherigen Position aus weiter
  // schiebt
  static uint16_t start_state= 1;
  static uint16_t lfsr= 1;
  static char first = 0;

  uint16_t lsb;

  if (first== 0)
  {
    first= 1;
    start_state= random_startwert;
    lfsr= start_state;
  }

  lsb = lfsr & 1;                        // niederwertigstes Bit des Schieberegisters
  lfsr = lfsr >> 1;                      // das Schieberegister, eine Stelle nach rechts
  if (lsb) { lfsr ^= 0xb400; }           // wenn LSB gesetzt, XOR-Togglemaske auf SR anwenden

  return (lfsr % rmax);
}



/* ----------------------------------------------------------
                          o_draw

     zeichnet den Buchstaben 'o' in einen Stein (wird be-
     noetigt umd die 4 siegreichen Steine anzuzeigen)
   ---------------------------------------------------------- */
void o_draw(uint8_t kx, uint8_t ky)
{
  if (c4_winner == human)
  {
    textcolor= rgbfromvalue(0x00,0x75,0x00);
  }
  else
    textcolor= rgbfromvalue(0x80,0xff,0x80);
  putcharxy((kx*17)+xofs+9, (ky*17)+yofs+8, 'o');
}


/* ----------------------------------------------------------
                          coin_draw

     zeichnet einen Stein an den Spielffeldkoordinaten
     kx, ky

     Uebergabe:
        ky, ky  : Spielfeldkoordinaten an denen der Stein
                  gezeichnet wird
        value   : Farbe des Steins (human, cpu, empty)
   ---------------------------------------------------------- */
void coin_draw(int8_t kx, int8_t ky, uint8_t value)
{
  int yo;

  yo= yofs;
  if (ky== -1) yo -= 3;

  if (value== human)
  {
    fillcircle( (kx*17)+xofs+12, (ky*17)+yo+12, 7, rgbfromega(14));
    circle( (kx*17)+xofs+12, (ky*17)+yo+12, 7, rgbfromvalue(0,0,0x80));
  }
  if (value== cpu)
  {
    fillcircle( (kx*17)+xofs+12, (ky*17)+yo+12, 7, rgbfromvalue(0xff,0,0));
    circle( (kx*17)+xofs+12, (ky*17)+yo+12, 7, rgbfromvalue(0,0,0x80));
  }

  if (value== empty)
  {
    fillcircle( (kx*17)+xofs+12, (ky*17)+yo+12, 7, rgbfromega(0));
  }
}

/* ----------------------------------------------------------
                       emptycoins_draw

     zeichnet alle Positionen des Spielfelds als leere
     Felder
   ---------------------------------------------------------- */
void emptycoins_draw(void)
{
  int kx, ky;

  for (ky= 0; ky < 6; ky++)
  {
    for (kx= 0; kx < 7; kx++)
    {
      fillcircle( (kx*17)+xofs+12, (ky*17)+yofs+12, 7, 0);
      circle( (kx*17)+xofs+12, (ky*17)+yofs+12, 7, rgbfromvalue(0,0,0x80));
    }
  }
}

/* ----------------------------------------------------------
                       emptyfield_draw

              zeichnet ein neues leeres Spielfeld
   ---------------------------------------------------------- */
void emptyfield_draw(uint16_t feldcol)
{

  fillcircle(9+xofs, 9+yofs, 9,feldcol);
  fillcircle(9+xofs, 99+yofs, 9,feldcol);
  fillcircle(117+xofs, 9+yofs, 9,feldcol);
  fillcircle(117+xofs, 99+yofs, 9,feldcol);

  fillrect(9+xofs,yofs, 117+xofs, 108+yofs, feldcol);
  fillrect(xofs, 9+yofs, 9+xofs, 99+yofs, feldcol);
  fillrect(115+xofs, 9+yofs, 127+xofs, 99+yofs, feldcol);

  emptycoins_draw();
}


/* ----------------------------------------------------------
                             c4_readf

     liest das Spielfeldarray und ueberprueft, ob angegebene
     Koordinaten im Spielefeld liegen
   ---------------------------------------------------------- */
uint8_t c4_readf(int8_t column, int8_t row)
{
  if ((column < 0) || (column > 6) || (row < 0) || (row > 5))
  {
    return outside;
  }
  else
  {
    return (c4_field[column][row]);
  }
}

/* ----------------------------------------------------------
                        c4_show_winning4

               zeigt die siegreichen 4 Steine an
   ---------------------------------------------------------- */

void c4_show_winning4(void)
{
  uint8_t x,y;

  for (y= 0; y< 6; y++)
  {
    for (x= 0; x< 7; x++)
    {
      if (c4_field[x][y]== 'P') o_draw(x,y);
      if (c4_field[x][y]== 'C') o_draw(x,y);
    }
  }
}

/* ----------------------------------------------------------
                        c4_mark_winning4

     markiert innerhalb des Spielfelds je nach Sieger die
     4 Steine in einer Reihe mit Zeichen 'P' oder 'C' damit
     diese bei einer Anzeige sichtbar gemacht werden koennen
   ---------------------------------------------------------- */
void c4_mark_winning4(uint8_t player)
{
  uint8_t x,y,z;
  uint8_t fmark;

  z= player;
  if (z== human) fmark= 'P'; else fmark= 'C';
  for (y= 0; y< 6; y++)
  {
    for (x= 0; x< 7; x++)
    {
      if ( x < 4 )
      {
        if ( (c4_field[x][y] == z) && (c4_field[x+1][y] == z) &&
             (c4_field[x+2][y] == z) && (c4_field[x+3][y] == z) )
        {
          // waagrechter Sieg
          c4_field[x][y]= fmark;   c4_field[x+1][y]= fmark;
          c4_field[x+2][y]= fmark; c4_field[x+3][y]= fmark;
          return;
        }

        if ( y < 3 )
        {
          if ( (c4_field[x][y] == z) && (c4_field[x+1][y+1] == z) &&
               (c4_field[x+2][y+2] == z) && (c4_field[x+3][y+3] == z) )
          {
            // diagonaler Sieg nach unten
            c4_field[x][y]= fmark;   c4_field[x+1][y+1]= fmark;
            c4_field[x+2][y+2]= fmark; c4_field[x+3][y+3]= fmark;
            return;
          }
        }

        if ( y > 2 )
        {
          if ( (c4_field[x][y] == z) && (c4_field[x+1][y-1] == z) &&
               (c4_field[x+2][y-2] == z) && (c4_field[x+3][y-3] == z) )
          {
            // diagonaler Sieg nach oben
            c4_field[x][y]= fmark;   c4_field[x+1][y-1]= fmark;
            c4_field[x+2][y-2]= fmark; c4_field[x+3][y-3]= fmark;
            return;
          }
        }
      }

      if ( y < 3 )
      {
        if ( (c4_field[x][y] == z) && (c4_field[x][y+1] == z) &&
             (c4_field[x][y+2] == z) && (c4_field[x][y+3] == z) )
        {
          // senkrechter Sieg
          c4_field[x][y]= fmark;   c4_field[x][y+1]= fmark;
          c4_field[x][y+2]= fmark; c4_field[x][y+3]= fmark;
          return;
        }
      }
    }
  }
}

/* -------------------------------------------------------
                         c4_chk_sanz

                      (check coin anz)
   testet, an den  Koordianten "x,y", ob eine Anzahl
   (sanz) Steine eines Spielers (player) waagrecht,
   diagonal oder senkrecht aufzufinden ist, die noch
   fuer die Vollendung eines 4ers ausreicht.

   Das Flag checkto4 bestimmt, ob nach einer Stellung ge-
   sucht wird, die zum sofortigen Sieg fuehrt.

   Eine gefundene Stellung wird innerhalb der Funktion
   im Flag checkfl gespeichert

   Uebergabe:
          x,y    : Koordinaten im Spielefeld, ab der ge
                   testet wird
          sanz   : Anzahl Steiner in einer Reihe, nach der
                   gesucht wird
        player   : Spieler, fuer den gesucht wird
        checkto4 : Test auf Stellung die zum sofortigen
                   Sieg fuehrt

   Rueckgabe:      1 wenn gefunden, 0 wenn nicht
   ------------------------------------------------------- */
uint8_t c4_chk_sanz(int8_t x, int8_t y, int8_t sanz, uint8_t player, uint8_t checkto4)
{
  int8_t  i,j,k;
  int8_t  sum1,  sum2,  sum3,  sum4,
          sum12, sum22, sum32, sum42;

  uint8_t contra_player;
  uint8_t checkfl;

  checkfl= 0;

  if (player == human)        // Gegner dieses Checks bestimmen
  {
    contra_player = cpu;
  }
  else
  {
    contra_player = human;
  };

  for (k= 0; k<= 3; k++)
  {
    sum1 = 0; sum2 = 0; sum3 = 0; sum4 = 0;
    sum12 = 0; sum22 = 0; sum32 = 0; sum42 = 0;

    for(j= 0; j<= 3; j++)
    {
      if (c4_readf(x-k+j,y) == player) sum1++;
      if (c4_readf(x,y-k+j) == player) sum2++;
      if (c4_readf(x-k+j,y-k+j) == player) sum3++;
      if (c4_readf(x+k-j,y-k+j) == player) sum4++;
      if (c4_readf(x-k+j,y) == contra_player) sum12++;
      if (c4_readf(x,y-k+j) == contra_player) sum22++;
      if (c4_readf(x-k+j,y-k+j) == contra_player) sum32++;
      if (c4_readf(x+k-j,y-k+j) == contra_player) sum42++;
      if (c4_readf(x-k+j,y) == outside) sum12++;
      if (c4_readf(x,y-k+j) == outside) sum22++;
      if (c4_readf(x-k+j,y-k+j) == outside) sum32++;
      if (c4_readf(x+k-j,y-k+j) == outside) sum42++;
    }

    if ((sum1 >= sanz) && (sum12 == 0)) checkfl = 1; else
    if ((sum2 >= sanz) && (sum22 == 0)) checkfl = 1; else
    if ((sum3 >= sanz) && (sum32 == 0)) checkfl = 1; else
    if ((sum4 >= sanz) && (sum42 == 0)) checkfl = 1;

    if ((checkfl) && (checkto4))
    {
      sum12 = 0;
      sum22 = 0;
      sum32 = 0;
      sum42 = 0;
      c4_field[x][y] = player;
      colf[x]--;

      for(j= 0; j<= 3; j++)
      {
        if ((sum1 >= sanz) && (c4_readf(x-k+j,y) == empty) && (c4_readf(x-k+j,colf[x-k+j]+1) == empty)) sum12++;
        if ((sum2 >= sanz) && (c4_readf(x,y-k+j) == empty) && (c4_readf(x,colf[x]+1) == empty)) sum22++;
        if ((sum3 >= sanz) && (c4_readf(x-k+j,y-k+j) == empty) && (c4_readf(x-k+j,colf[x-k+j]+1) == empty)) sum32++;
        if ((sum4 >= sanz) && (c4_readf(x+k-j,y-k+j) == empty) && (c4_readf(x+k-j,colf[x+k-j]+1) == empty)) sum42++;
      }
      if ((sum12 == 1) || (sum22 == 1) || (sum32 == 1) || (sum42 == 1)) checkfl = 0;
      colf[x]++;
      c4_field[x][y] = empty;
    }
  }
  return checkfl;
}

/* -------------------------------------------------------
                         c4_cpu_turn

     berechnet den Computerzug und fuegt entsprechend
     einen Stein ins Spielefeld ein.

     Rueckgabe:   o = Spiel ist noch nicht beendet
                  2 = Computer hat gewonnen
                  3 = Spiel endete unentschieden
   ------------------------------------------------------- */
uint8_t c4_cpu_turn(void)
{
  int8_t  i, j, k;
  int8_t  column;
  uint8_t cnt;
  int32_t x;
  int32_t chance[7];

  // Gewichtung der einzelnen Spalten + einem zusaetzlichen
  // Pseudo-Zufalls Element
  chance[0] = 13 + my_random(4);
  chance[1] = 13 + my_random(4);
  chance[2] = 16 + my_random(4);
  chance[3] = 16 + my_random(4);
  chance[4] = 16 + my_random(4);
  chance[5] = 13 + my_random(4);
  chance[6] = 13 + my_random(4);

  for (i=0; i<= 6; i++)
  {
    if (colf[i] < 0)  chance[i] = chance[i]-30000;
  }

  for (i=0; i<= 6; i++)
  {
    // ist eine Position vorhanden, mit der mit dem naechsten Stein gewonnen werden kann?
    if (c4_chk_sanz(i,colf[i],3,cpu,0)) chance[i] = chance[i] + 20000;

    // kann der Gegner mit dem naechsten Stein gewinnen ?
    if (c4_chk_sanz(i,colf[i],3,human,0)) chance[i] = chance[i] + 10000;

    // 3-er Reihe darueber (contra_player)?
    if (c4_chk_sanz(i,colf[i]-1,3,human,0)) chance[i] = chance[i] -4000;

    // 3er-Reihe darueber (CPU)
    if (c4_chk_sanz(i,colf[i]-1,3,cpu,0)) chance[i] = chance[i] -200;

    // 2er Reihe erweitern
    if (c4_chk_sanz(i,colf[i],2,human,0)) chance[i] = chance[i] + 50 + my_random(3);

    // dto. aber nur, wenn nicht gleich ausgeschaltet werden kann
    if ((c4_chk_sanz(i,colf[i],2,cpu,1)) && (colf[i] > 0))
    {
      c4_field[i][colf[i]] = cpu;
      colf[i]--;
      cnt = 0;

      for(j= 0; j<= 6; j++)
      {
        if(c4_chk_sanz(j,colf[j],3,cpu,0) ) cnt++;
      }

      if (cnt == 0) chance[i] = chance[i] + 60 + my_random(2); else chance[i] = chance[i] - 60;
      colf[i]++;
      c4_field[i][colf[i]] = empty;
    }

    // diverse Stellungsbewertungen (wenn etwas darueber / darunter liegt, wenn etwas erweitert
    // werden kann

    if (c4_chk_sanz(i,colf[i]-1,2,human,0) ) chance[i] = chance[i] -10;

    if (c4_chk_sanz(i,colf[i]-1,2,cpu,0) ) chance[i] = chance[i] -8;

    if (c4_chk_sanz(i,colf[i],1,human,0) ) chance[i] = chance[i] + 5 + my_random(2);

    if (c4_chk_sanz(i,colf[i],1,cpu,0) ) chance[i] = chance[i] + 5 + my_random(2);

    if (c4_chk_sanz(i,colf[i]-1,1,human,0) ) chance[i] = chance[i] -2;

    if (c4_chk_sanz(i,colf[i]-1,1,cpu,0) ) chance[i] = chance[i] +1;

    // Spielemuster, die zum Sieg der CPU fuehren kann
    if ((c4_chk_sanz(i,colf[i],2,cpu,1) ) && (colf[i] > 0))
    {
      c4_field[i][colf[i]] = cpu;
      colf[i]--;
      for (k= 0; k<= 6; k++)
      {
        if ((c4_chk_sanz(k,colf[k],3,cpu,0) ) && (colf[k] > 0))
        {
          c4_field[k][colf[k]] = human;
          colf[k]--;
          for(j=0; j<=6;j++)
          {
            if (c4_chk_sanz(j,colf[j],3,cpu,0) ) chance[i] = chance[i] + 2000;
          }
          colf[k]++;
          c4_field[k][colf[k]] = empty;
        }
      }
      colf[i]++;
      c4_field[i][colf[i]] = empty;
    }

    // Spielemuster, die zum Sieg des Players fuehren kann
    if ((c4_chk_sanz(i,colf[i],2,human,1) ) && (colf[i] > 0))
    {
      c4_field[i][colf[i]] = human;
      colf[i]--;
      for (k= 0; k<= 6; k++)
      {
        if ((c4_chk_sanz(k,colf[k],3,human,0) ) && (colf[k] > 0))
        {
          c4_field[k][colf[k]] = cpu;
          colf[k]--;
          for(j= 0;j<= 6; j++)
          {
            if (c4_chk_sanz(j,colf[j],3,human,0) ) chance[i] = chance[i] + 1000;
          }

          colf[k]++;
          c4_field[k][colf[k]] = empty;
        }
      }
      colf[i]++;
      c4_field[i][colf[i]] = empty;
    }

    // Test auf bestimmte Muster, ob bei Aktion im Array, zu einem fuer die CPU negativen
    // Spielemuster kommen kann
    if ((c4_chk_sanz(i,colf[i]-1,2,human,1) ) && (colf[i] > 1))
    {
      c4_field[i][colf[i]] = human;
      colf[i]--;
      for (k= 0; k<= 6; k++)
      {
        if ((c4_chk_sanz(k,colf[k]-1,3,human,0) ) && (colf[k] > 0))
        {
          c4_field[k][colf[k]] = cpu;
          colf[k]--;
          for(j= 0; j<= 6; j++)
          {
            if (c4_chk_sanz(j,colf[j]-1,3,human,0) ) chance[i] = chance[i] - 500;
          }
          colf[k]++;
          c4_field[k][colf[k]] = empty;
        }
      }
      colf[i]++;
      c4_field[i][colf[i]] = empty;
    }
  }

  column = 0;
  x = -10000;
  for (i= 0; i<= 6; i++)
  {
    if (chance[i] > x)
    {
      x = chance[i];
      column = i;
    }
  }

  c4_field[column][colf[column]] = cpu;               // setzt den vom Computer errechneten Stein ins Array

  coin_draw(column, colf[column], cpu);

  colf[column]--;

  if (c4_chk_sanz(column,colf[column]+1,4,cpu,0) )
  {
    // Computer hat gewonnen
    return cpu;
  }
  if ((colf[0] == -1) && (colf[1] == -1) && (colf[2] == -1) &&
      (colf[3] == -1) && (colf[4] == -1) && (colf[5] == -1) &&
      (colf[6] == -1))
  {
    // Unentschieden
    return 3;
  }
  return 0;
}

/* -------------------------------------------------------
                       c4_player_turn

     fuehrt den Spielerzug durch und ueberprueft die
     Gueltigkeit des Zuges (ist Schacht voll?). An-
     schliessender Test, ob Spieler gewonnen hat. Hat
     Spieler nicht gewonnen, wird automatisch ein
     Computerzug aufgerufen

     Rueckgabe:   100:  Einwurfschacht ist voll, Spieler
                        muss anderen Schacht waehlen
                    1:  Spieler gewinnt
                    2:  Computer gewinnt
                    3:  Unentschieden
   ------------------------------------------------------- */
int8_t c4_player_turn(int8_t column)
{
  uint8_t playerwin;
  int8_t i, sum;

  playerwin= 0;
  if (colf[column] == -1)
  {
    // Schacht ist voll
    return 100;
  }
  else
  {
    c4_field[column][colf[column]] = human;
    coin_draw(column, colf[column],human);

    colf[column] = colf[column] - 1;
    if (c4_chk_sanz(column,colf[column]+1,4,human, 0))
    {
      playerwin= 1;
      // Spieler gewinnt\n");
      return human;
    }

    sum= 0;
    for (i= 0; i< 7; i++) sum += colf[i];
    if (sum == -8)
    {
      // Unentschieden
      return 3;
    }

    if (playerwin== 0)
    {
      return c4_cpu_turn();
    }
  }
}

/* -------------------------------------------------------
                     get_player_col

     waehlt mittels Tasten den Schacht, in den der Stein
     eingeworfen wird
   ------------------------------------------------------- */
char get_player_col(void)
{

  fillrect(0,0, 127,yofs-1, rgbfromega(0));
  coin_draw(selectpos, -1, human);

  while(!(is_keydwn()))
  {
    if (is_keyleft())
    {
      if (selectpos> 0)
      {
        delay(50);
        while(is_keyleft());
        delay(50);
        coin_draw(selectpos, -1, empty);
        selectpos--;
        coin_draw(selectpos, -1, human);
      }
    }
    if (is_keyright())
    {
      if (selectpos< 6)
      {
        delay(50);
        while(is_keyright());
        delay(50);
        coin_draw(selectpos, -1, empty);
        selectpos++;
        coin_draw(selectpos, -1, human);
      }
    }
  }
  coin_draw(selectpos, -1, 3);
  delay(200);

  return selectpos;
}

/* -------------------------------------------------------
                        score_show

     zeigt den Spielstand zwischen Computer und Spieler
     an.
   ------------------------------------------------------- */
void score_show(void)
{
  uint8_t zstr[7];
  uint8_t l;
  uint8_t x;


  bkcolor= 0;
  clrscr();
  fillellipse(64,23, 50,20, rgbfromvalue(0xff, 0x7a, 0x2b));
  ellipse(64,23, 50,20, rgbfromega(8));
  ellipse(64,23, 48,18, rgbfromega(15));
  textsize= 1;
  textcolor= rgbfromvalue(0xff, 0x7a, 0x2b);
  textcolor= 0;
  outtextxy(27,17, 0, "Score");
  textsize= 0;

  textcolor= rgbfromega(14);
  outtextxy(20, 53, 0, "Player:");
  itoa(playerscore, zstr, 10);
  outtextxy(92, 53, 0, zstr);

  textcolor= rgbfromega(12);
  outtextxy(20, 65, 0, "CPU  :");
  itoa(cpuscore, zstr, 10);
  outtextxy(92, 65, 0, zstr);
}

/* ---------------------------------------------------------------------------
                                    M A I N
   --------------------------------------------------------------------------- */
int main(void)
{
  uint16_t vgblue;
  uint16_t cnt;
  int8_t x,y;
  uint8_t zug;
  uint8_t ch;
  int8_t  erg;

  lcd_init();
  button_init();

  textsize= 0;
  setfont(0);
  lcd_orientation(0);
  lcd_enable();
  vgblue= rgbfromvalue(0x40,0x20,0xff);
  bkcolor= rgbfromega(15);
  clrscr();
  gotoxy(0,0);
  pcx256_show(0,0, &vg_logo[0], &vg_logopal[0]);

  // Warten auf Tasten und dabei einen zufaelligen Startwert fuer den
  // Pseudozufallszahlengenerator erzeugen
  random_startwert= 0;
  do
  {
    random_startwert++;
  }while( !(is_keydwn()));
  delay(50);
  while( is_keydwn());
  delay(50);

  while(1)
  {
    bkcolor= 0;
    clrscr();
    score_show();
    while( !(is_keydwn()));
    delay(50);
    while( is_keydwn());
    delay(50);

    memset(c4_field, empty, 42);            // Spielfeld mit 0 fuellen (7*6= 42 Elemente)
    memset(colf, 5, 7);                     // max. einfuellbare Steine der einzelnen
                                            // Schaechte= 5 (0..5 = 6 Steine)
    clrscr();
    emptyfield_draw(vgblue);
    emptycoins_draw();

    do                                      // so lange das Spiel nicht beendet ist
    {
      do                                    // solange bis in freien Schacht geworfen
      {
        zug= get_player_col();
        erg= c4_player_turn(zug);
      } while (erg == 100);

      if (erg > 0) c4_mark_winning4(erg);
    } while(erg == 0);
    c4_winner= erg;
    fillrect(0,0, 127,yofs-1, rgbfromega(0));
    coin_draw(selectpos, -1, empty);
    c4_show_winning4();
    textcolor= rgbfromega(1);
    if (erg == human) { textcolor= rgbfromega(14); my_outtextxy(4,6,"Spieler gewinnt"); playerscore++;}
    if (erg == cpu)   { textcolor= rgbfromega(12); my_outtextxy (4,6,"  CPU gewinnt"); cpuscore++;}
    if (erg == 3)     { textcolor= rgbfromega(14); my_outtextxy(4,6,    " Unentschieden");}
    delay(3000);
    selectpos= 3;
  }
}
